<?php

// DB_result_base.class.php - Base class for classes dealing with result sets from database queries
// Replacement for old DB_result which was incorporated in each DB module
//
// $Id: DB_result.class.php 1424 2008-12-29 16:09:36Z tim $
//
// (c)2001-2008 Tim Jackson (tim@timj.co.uk)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
 * This class is used for handling database query results. It attempts vaguely 
 * to maintain a public interface similar to that of the equivalent PEAR class 
 * of the same name.
 */
class DB_result_base {

	var $_dbh;
	var $_rr;
	var $pages=1;	// total pages for this query
	
	// Constructor
	function DB_result_base (&$dbh, $rr) {
		$this->_dbh = &$dbh;
		$this->_rr = $rr;
	}

	/**
	 * Get the number of rows in the result set
	 *
	 * @return int
	 */
	function numRows () { return PHPOF_NOTSUPPORTED; }

	/**
	 * Fetch the next result as an associative array
	 */
	function fetchRow () { return PHPOF_NOTSUPPORTED; }

	/**
	 * Return the number of rows affected by the previous operation
	 *
	 * @return int
	 */
	function affectedRows () { return PHPOF_NOT_SUPPORTED; }
	
	/**
	 * Get the primary key of the last row inserted into the database
	 *
	 * @param string $table_name
	 * @return string  Primary key
	 */
	function insertedId ($table_name='') { return PHPOF_NOTSUPPORTED; }
	
	/**
	 * Get an associative array containing all results from the result set
	 * 
	 * This function is similar to DBTable::getArray and can be very useful for
	 * creating arrays ready to be turned into an HTML SELECT using 
	 * XArray::CreateSelect
	 *
	 * @param string $key  Column in the result set to use as key in associative array
	 * @param string $value  Column in the result set to populate the values in the associative array
	 * @return array  The result set in an array
	 */
	function getArray ($key, $value=PHPOF_ARRAY_LINEAR) {
		while ($result = $this->fetchRow()) {
			if ($value == PHPOF_ARRAY_LINEAR) {
				$array[] = $result[$key];
			} else {
				$array[$result[$key]] = $result[$value];
			}
		}
		if (isset($array)) {
			return $array;
		}
		return false;
	}
	
	// $Index is the name of the column to be used as an index, or PHPOF_USE_KEY
	// $Expression is an expression to be evaluated in the context of a DBRow object
	function getArrayWithDN($RowObject, $Index=PHPOF_USE_KEY, $Expression=PHPOF_USE_DEFAULT_DN, $OrderBy=PHPOF_USE_DEFAULT_ORDER) {
		
		$ReturnArray = null;
		
		if ($Expression != PHPOF_USE_DEFAULT_DN) {
			$RowObject->DescriptiveNameExpression = $Expression;
		}
		
		if (empty($RowObject->DescriptiveNameExpression)) trigger_error("DB_result::getArrayDN() called with request to use DBRow DescriptiveName (DN) as result value, but DBRow object does not have a DescriptiveNameExpression defined");
		
		if ($Index == PHPOF_USE_KEY) {
			$Index = $RowObject->_table->_key;
		}
		
		while ($this->fetchIntoObject($RowObject)) {
			$ReturnArray[$RowObject->$Index] = $RowObject->DescriptiveName;
		}
		return $ReturnArray;
	}

	/**
	 * Retrieve the next result from from a previously-performed search into an object
	 * 
	 * This function gets the next result and fills a row object with it. You 
	 * need to pass in a DBRow object result_row which will be filled with the
	 * results. For obvious reasons, the object you pass in really ought to have
	 * been initialised with the appropriate table name, although PHPOF does
	 * not enforce this.
	 * 
	 * Example
		
	 * // Assume we have an SQL table 'products' like so:
	 * // +------+-------------------+
	 * // | code | product_name      |
	 * // +------+-------------------+
	 * // | FB1  | Foo Bar           |
	 * // | FB2  | Deluxe Foo        |
	 * // | XY1  | Xygluprenia       |
	 * // | BB1  | Bob's Biscuit Bar |
	 * // +------+-------------------+
	 * 
	 * $products = new DBTable('products');
	 * $product = new DBRow($products);
	 * 
	 * $results = $products->BasicSearchLike('product_name','Bar');
	 * 
	 * while ($results->fetchIntoObject($product)) {
	 *    print "Product code: $product->code\n";
	 *    print "Product name: $product->product_name\n\n";
	 * }
	 * 
	 *  output:
	 * Product code: FB1
	 * Product name: Foo Bar
	 * 
	 * Product code: BB1
	 * Product name: Bob's Biscuit Bar
	 *
	 * @param DBRow $object_to_fill
	 * @return bool  Whether the fetch was successful
	 */
	function fetchIntoObject (&$object_to_fill) {
		global $PHPOF_AUDIT_LOGGING;

		if (!is_object($object_to_fill)) trigger_error('DB_result::fetchIntoObject() was not passed an object',E_USER_ERROR);
		if (!empty($object_to_fill->_Hook_FetchIntoObject_pre)) eval('$object_to_fill->'.$object_to_fill->_Hook_FetchIntoObject_pre);
		
		if ($row = $this->fetchRow()) {
			foreach ($row as $field => $value) {
				$object_to_fill->$field = $value;
			}
			if ($PHPOF_AUDIT_LOGGING) $object_to_fill->_storeAuditData();
			
			// Set row object descriptive name
			$object_to_fill->setDescriptiveName();
			
			$object_to_fill->resolveBinaryFields();
			
			// Call post-hook
			if (!empty($object_to_fill->_Hook_FetchIntoObject_post)) eval('$object_to_fill->'.$object_to_fill->_Hook_FetchIntoObject_post);
			return true;
		} else {
			return false;
		}
	}
	
}
